@@ -1,5 +1,34 @@
 pulkrob.deviance <-
 function(model, catvars) {
+  epi.cp.pulkrob <- function(dat){
+    # adapted from epiR::epi.cp() by Mark Stevenson et al
+    ndat <- data.frame(id = 1:nrow(dat), dat)
+    if (!is.null(dim(ndat[, ncol(ndat):2]))) {
+      # Add an indicator variable for covariate patterns:
+      ndat$indi <- apply(X = ndat[, ncol(ndat):2], MARGIN = 1, FUN = function(x) as.factor(paste(x, collapse = "")))
+      # Order the data according to the indicator variable:
+      ndat <- ndat[order(ndat$indi),]
+      # Create a variable that indicates all the cases of each covariate pattern:
+      cp.id <- tapply(ndat$id, ndat$indi, function(x) paste(x, collapse = ","))
+      # Create a data frame of unique covariate patterns:
+      cp <- unique(ndat[, 2:ncol(ndat)])
+      n <- as.numeric(unlist(lapply(strsplit(cp.id, ","), length)))
+      id <- tapply(ndat$id, ndat$indi, function(x) (x)[1])
+      lookup <- data.frame(id = 1:length(n), indi = row.names(id))
+      cov.pattern <- data.frame(id = 1:length(n), n, cp[,-ncol(cp)])
+      rownames(cov.pattern) <- rownames(cp)
+      # Create a vector with the covariate pattern for each case:
+      id <- lookup$id[match(ndat$indi, lookup$indi)]
+    } else {
+      #ndat <- ndat[order(ndat[2]), ]
+      cp.id <- tapply(ndat$id, ndat[2], function(x) paste(x, collapse = ","))
+      cp <- unique(ndat[, 2:ncol(ndat)])
+      n <- as.numeric(unlist(lapply(strsplit(cp.id, ","), length)))
+      cov.pattern <- data.frame(id = 1:length(n), n, cp)
+      id <- cov.pattern$id[match(as.vector(as.matrix(ndat[2])), cov.pattern$cp)]
+    }
+    list(cov.pattern = cov.pattern, id = id)
+  }
   if (class(model) == "polr") {
     yhat <- as.data.frame(fitted(model))
   } else if (class(model) == "clm") {
@@ -11,7 +40,7 @@ function(model, catvars) {
   METHOD <- "Pulkstenis-Robinson deviance test"
   covars <- model$model[-1]
   covars <- covars[names(covars) %in% catvars]
-  covpat <- epi.cp(covars)
+  covpat <- epi.cp.pulkrob(covars)
   yhat$score <- apply(sapply(1:ncol(yhat), function(i) { yhat[, i] * i }), 1, sum)
   yhat <- cbind(id=1:nrow(yhat), yhat, covpat=covpat$id)
   medians <- cbind(covpat=covpat$cov.pattern$id,
